import { describe, it } from '@std/testing/bdd'
import { assertEquals, assertRejects } from '@std/assert'
import { QueryEvent } from './QueryEvent.ts'
import { LocalTimedActivity } from '../adapters/jasd/tests/fixtures.ts'
import { Activity } from '../adapters/jasd/types/appapi.dto.types.ts'
import { GraphQLError } from 'npm:graphql@16.10.0'

describe('QueryEvent', () => {
  it('throws when no event can be fetched', () => {
    const jasdGatewayMock = {
      fetchActivity() {
        return Promise.reject(new GraphQLError(''))
      },
    }

    const sut = new QueryEvent(jasdGatewayMock, console)

    assertRejects(() => sut.execute({ id: 'iDoesNotExist' }), GraphQLError)
  })

  it('works for a jasd api response', async () => {
    const jasdGatewayMock = {
      fetchActivity(): Promise<Activity> {
        return Promise.resolve(LocalTimedActivity)
      },
    }

    const expectedEvent = {
      data: {
        endDate: '2023-09-25T00:00:00+02:00',
        eventDetails: {
          description:
            '<p>Beim Reparieren elektrischer Geräte kommt früher oder später oft der Punkt, an dem gelötet werden muss. In diesem Workshop können Sie lernen und üben, richtig mit Lötkolben und Zinn umzugehen. Es ist kein Vorwissen nötig. Wenn die Grundlagen geklärt sind, können wir uns gemeinsam zu verschiedenen Anwendungsfeldern vortasten - je nachdem wofür Sie sich besonders interessiert. Dafür können Sie auch eigene Lötprojekte mitbringen (Beispiele: Kabel mit Wackelkontakt, z.B. von Kopfhörern oder Audiokabeln, kleine Geräte diverser Art wie Akku-Fahrradbeleuchtung, Lichterketten oder auch Stirnlampen/Taschenlampen).</p>',
          externalLink:
            'https://www.vhs-leipzig.de/p/normale-kurse/kunst-kultur-kreativitaet/kreatives-gestalten/metall-und-schmuckgestaltung/einstiegskurs-loeten-495-C-C2A1001K',
          meetingLink: undefined,
        },
        id: '1234',
        imageUrl: 'https://gemeinschaftswerk-nachhaltigkeit.de/app/api/v1/files/activities/act6.jpg',
        isRemote: false,
        location: {
          city: 'Leipzig',
          street: 'Löhrstr.',
          streetNo: '3-7',
          zipCode: '04105',
        },
        organisationName: 'Volkshochschule Leipzig',
        startDate: '2023-09-25T00:00:00+02:00',
        title: 'Einstiegskurs Löten',
      },
    }

    const sut = new QueryEvent(jasdGatewayMock, console)

    const result = await sut.execute({ id: '4160' })

    assertEquals(result, expectedEvent)
  })
})
